package com.sunlabs.archsim.fleet.ships.testing;

import com.sunlabs.archsim.base.simulation.Command;
import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.common.Payload;
import com.sunlabs.archsim.gasp.Function;

/**
 * The function returns values as listed in the FunctionStep array.
 * The values are taken from the array in the circular manner, starting
 * with the element at index 0.
 * At each step,all commands associated with that step are executed.
 * The commands can, for example, be used to enable or disable TriggerActions.
 * The function can return either a Payload object or a Data object, depending on 
 * a boolean switch. In testing, typically data sources generate Payload objects.
 * 
 * @author ib27688
 *
 */

public class ScriptedFunction extends Function {

	protected int currentStep;
	
	protected FunctionStep[] steps;
	public final int totalSteps;
	protected boolean payloadOut;
	
	public ScriptedFunction(String name, FunctionStep[] fs, boolean po) {
		super(name);
		steps= fs;
		currentStep= 0;
		totalSteps= (steps==null)? 0 : steps.length;
		payloadOut= po;
	} // ScriptedFunction
	
	/**
	 * Default is to creat a payload output.
	 * @param name
	 * @param fs
	 */
	public ScriptedFunction(String name, FunctionStep[]fs) {
		this(name, fs, true);
	}
	
	public Object value() {
		if (steps != null) {
			// first execute all commands
			if (steps[currentStep].commands != null) {
				for (Command c : steps[currentStep].commands) {
					if (c != null) c.execute(null);
				}
			}
			// remember the data to return
			Data d= steps[currentStep].data;
			// increase the step count
			currentStep= (currentStep + 1) % totalSteps;
			// return the data
			if (payloadOut) {
				Payload p= new Payload();
				p.data= d;
				return p;
			} 
			else {
				return d;
			}
		}
		else {
			// nothing was specified
			return null;
		}
	} // value
	
	public int getCurrentStep() {
		return currentStep;
	} // getCurrentStep;
	
} //ScriptedFunction
