/*
 * Copyright ? 2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 *
 */

/*
 * Created on October 26, 2005, 4:58 PM
 */

package com.sunlabs.archsim.fleet.ships.tokenadder;


import java.util.List;
import java.util.Iterator;

import com.sunlabs.archsim.runner.Runner;
import com.sunlabs.archsim.runner.OperationLoader;
import com.sunlabs.archsim.base.hierarchy.Entities;
import com.sunlabs.archsim.engine.Director;


import com.sunlabs.archsim.fleet.ships.testing.DataSink;

import com.sunlabs.archsim.fleet.common.Data;
import com.sunlabs.archsim.fleet.ships.testing.PayloadSource;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.framework.Test;
import junit.textui.TestRunner;

/**
 *
 * @author mh193489
 */
public class TestTokenAdder extends TestCase {
    
    private static String fileName= "src/" + TestTokenAdder.class.getPackage().getName().replace('.', '/') + "/TokenAdderTestScript.xml";
    
    private static Director director;
    
    /* sources and sinks */
    
    private static final String DATA1_SOURCE= "Data1Source";
    private static final String DATA2_SOURCE= "Data2Source";
    private static final String DATA_SINK= "DataSink";
    private static final String TOKEN1_SINK= "Token1Sink";
    private static final String TOKEN2_SINK= "Token2Sink";
    private static final String TOKEN_OUT_SOURCE= "TokenSource";
    
    /* reference to the DataSink SHIP for result inspection */
    private DataSink dataSink;
    private DataSink token1Sink;
    private DataSink token2Sink;
    
    public void setUp() {
        
        
        // for sum test
        PayloadSource.add(DATA1_SOURCE, new Data(1));
        PayloadSource.add(DATA2_SOURCE, new Data(2));
        PayloadSource.add(TOKEN_OUT_SOURCE, new Data(true));
        
        // for overflow test
        PayloadSource.add(DATA1_SOURCE, new Data(0));
        PayloadSource.add(DATA2_SOURCE, new Data(AdderDefaults.OVERFLOW));
        PayloadSource.add(TOKEN_OUT_SOURCE, new Data(true));
        
        // for underflow test
        PayloadSource.add(DATA1_SOURCE, new Data(0));
        PayloadSource.add(DATA2_SOURCE, new Data(AdderDefaults.UNDERFLOW));
        PayloadSource.add(TOKEN_OUT_SOURCE, new Data(true));
        
        // for OOB on inputs test
        PayloadSource.add(DATA1_SOURCE, new Data(1));
        PayloadSource.add(DATA2_SOURCE, new Data(false));
        PayloadSource.add(TOKEN_OUT_SOURCE, new Data(true));
        
        PayloadSource.add(DATA1_SOURCE, new Data(false));
        PayloadSource.add(DATA2_SOURCE, new Data(2));
        PayloadSource.add(TOKEN_OUT_SOURCE, new Data(true));
        
        
        
        // initialize the tests
        director= Director.getInstance();
        
        
        OperationLoader ldr= new OperationLoader();
        List ops= ldr.load(fileName);
        
        Runner r= new Runner();

        r.perform(ops);

        /* get the reference to current DataSink SHIP instance */
        dataSink= (DataSink)Entities.find(DATA_SINK);
        token1Sink= (DataSink)Entities.find(TOKEN1_SINK);
        token2Sink= (DataSink)Entities.find(TOKEN2_SINK);

    }
    
    protected void tearDown() {
        // clean up after tests
        //director.closeJournal();
        director.masterClear();
    }
    
    public static Test suite() {
        return new TestSuite(TestTokenAdder.class);
    }
    
    public static void main(String[] args) {
        // run the tests
        TestRunner.run(TestTokenAdder.class);
    } // main
    
    public void testTestSetup() {
        // data sink
        assertNotNull(dataSink);
        assertNotNull(dataSink.receivedData);
        for (Iterator<Data> i= dataSink.receivedData.iterator(); i.hasNext();) {
            // check if data is not null
            Data data= i.next();
            assertNotNull(data);
            // check if data value is not null, in case data is valid
            if (data.valid) {
                assertNotNull(data.value);
            }
        }
        
    } 
    
    /* Tests for the data interface */
    
    public void testSum() {
        assertTrue(dataSink.receivedData.get(0).valid);
        assertEquals( ((Integer)dataSink.receivedData.get(0).value).intValue(), 3);
    }
    
    
    public void testOverflow() {
        assertFalse(dataSink.receivedData.get(1).valid);
        assertEquals(dataSink.receivedData.get(1).value, AdderDefaults.PLUS_INFINITY);
    }
    
    
    public void testUnderflow() {
        assertFalse(dataSink.receivedData.get(2).valid);
        assertEquals(dataSink.receivedData.get(2).value, AdderDefaults.MINUS_INFINITY);
    }
    
    public void testOOBonInputs() {
        // In1
        assertFalse(dataSink.receivedData.get(3).valid);
        assertEquals(dataSink.receivedData.get(3).value, AdderDefaults.OOB);
        // In2
        assertFalse(dataSink.receivedData.get(4).valid);
        assertEquals(dataSink.receivedData.get(4).value, AdderDefaults.OOB);
    }
    
    public void testOOBtokenIn() {
        assertTrue(token1Sink.receivedData.get(3).valid);
        assertFalse(token2Sink.receivedData.get(3).valid);
        
        assertFalse(token1Sink.receivedData.get(4).valid);
        assertTrue(token2Sink.receivedData.get(4).valid);
    }
    
}
